#!/usr/bin/python

#
# Copyright (C) 2008 by OpenMoko, Inc.
# Written by Willie Chen <willie_chen@openmoko.com>
#            Marek Lindner <marek@openmoko.com>
# All Rights Reserved
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import sys

from expos import gsm
from expos import bt
from expos import gps
from expos import brightness
from expos import suspend
from expos import time
from expos import sysinfo
from expos import wifi
from expos import profile


### TODO ###

## modules ##
# GSM -> needs Qtopia dbus call (holger)
# WiFi -> needs network manager (raster)
# Bluetooth -> DONE
# GPS -> DONE
# Backlight -> DONE
# Suspend -> DONE
# Profile -> DONE
# Time -> DONE
# System Info -> DONE
##

## misc ##
# slow startup because of etk loading
# load backlight settings on boot
##


class exposure():
	def __init__(self, etk, edje, ecore, name="exposure"):
		self.name = name
		self.etk = etk
		self.edje = edje
		self.ecore = ecore

	def run(self):

		### load and initialize modules ###

		self.mygsm = gsm()
		self.mybt = bt()
		self.mygps = gps()
		self.mywifi = wifi(self.etk)
		self.mybrightness = brightness()
		self.mysuspend = suspend()
		self.mytime = time()
		self.mysysinfo = sysinfo(self.etk)
		self.myprofile = profile()

		self.loaded_modules = [self.mygsm, self.mywifi, self.mybt, self.mygps, self.mybrightness, self.myprofile, self.mysuspend, self.mytime, self.mysysinfo]


		### window init ###

		if self.ecore.evas.engine_type_supported_get("software_x11_16"):
			self.window = self.ecore.evas.SoftwareX11_16(w=480, h=575)
		else:
			self.window = self.ecore.evas.SoftwareX11(w=480, h=575)

		self.window.callback_delete_request = self.window_delete_request
		self.window.title = "Exposure"

		self.edje_file="/usr/share/exposure/exposure.edj"
		self.edj = self.edje.Edje(self.window.evas, file=self.edje_file, group="exposure")
		self.edj.size = self.window.evas.size
		self.window.data["edje"] = self.edj
		self.edje.frametime_set(1.0/30)

		self.edj.signal_callback_add("mouse,up,1", "arrow-event", self.arrow_clicked)
		self.edj.show()

		self.mysysinfo.edj_object = self.edj
		self.mywifi.edj_object = self.edj
		self.mywifi.edje_file = self.edje_file
		self.mywifi.edje = self.edje.Edje
		self.mywifi.canvas = self.window.evas


		### gui setup ###

		self.item_list = self.etk.ListModel()

		for index, item in enumerate(self.loaded_modules):
			self.item_list.append([item.name, self.label_text(item.name, "")])
			item.button = self.item_list[index]
			item.label_text = self.label_text
			item.timer_add = self.ecore.timer_add
			item.init_label()

		## add empty item so that the last item can scroll into the visible screen
		self.item_list.append(["scroll_helper", ""])

		self.lst = self.etk.List(self.item_list, selectable=True, animated_changes=True, row_height=80, columns=[(480, self.etk.KineticTextRenderer(slot=1, click=self.clicked),True)])

		self.mainmenu_win = self.etk.Embed(self.window.evas)
		self.mainmenu_win.add(self.lst)
		self.mainmenu_win.show_all()
		self.edj.part_swallow("main-menu", self.mainmenu_win.object)

		self.sysinfo_win = self.etk.Embed(self.window.evas)
		self.sysinfo_win.add(self.mysysinfo.VBox)
		self.sysinfo_win.show_all()
		self.edj.part_swallow("sysinfo", self.sysinfo_win.object)

		self.wifi_win = self.etk.Embed(self.window.evas)
		self.wifi_win.add(self.mywifi.scrolled_view)
		self.wifi_win.show_all()
		self.edj.part_swallow("wifi", self.wifi_win.object)

		self.window.show()
		self.ecore.main_loop_begin()

		self.edj.part_unswallow(self.sysinfo_win.object)
		self.edj.part_unswallow(self.wifi_win.object)
		self.edj.part_unswallow(self.mainmenu_win.object)
		self.edj.delete()
		self.window.hide()
		self.ecore.main_loop_iterate()

	def window_delete_request(self, evas_obj):
		self.ecore.main_loop_quit()

	def clicked(self, button):
		for module in self.loaded_modules:
			if str(button).find(module.name) > -1:
				module.onclick()
				break

	def label_text(self, item, status):
		return "<font_size=48>" + item + "<font_size=24>" + '  ' + status

	def arrow_clicked(self, edje_obj, emission, source):
		self.edj.part_text_set("settings-text", "Settings")

